local _G = getfenv(0)

MessageEvent = AceLibrary("AceAddon-2.0"):new("AceHook-2.1", "AceConsole-2.0")

function MessageEvent:OnInitialize()
	self:RegisterChatCommand({"/mess", "/messevent"}, {
		type = "group",
		args = {
			show = {
				name = "show", type = "text",
				desc = "Enable event messages/links for the given frame(s).",
				validate = { "all", "1", "2", "3", "4", "5", "6", "7" },
				get = false,
				set = function(frame)
					self:EventsToggle(true, frame)
					self:Print("MessageEvent activated for ["..frame.."].")
				end,
			},
			hide = {
				name = "hide", type = "text",
				desc = "Disable event messages/links for the given frame(s).",
				validate = { "all", "1", "2", "3", "4", "5", "6", "7" },
				get = false,
				set = function(frame)
					self:EventsToggle(false, frame)
					self:Print("MessageEvent deactivated for ["..frame.."].")
				end,
			},
			report = {
				name = "report", type = "execute",
				desc = "Reports the hook status of all the chat frames.",
				func = function()
					for i = 1, 7 do
						local frame = _G["ChatFrame"..i]
						self:Print("ChatFrame"..i.." is hooked: ["..tostring(self:IsHooked(frame, "AddMessage")).."].")
					end
				end,
			},
		},
	}, "MESSAGEEVENT")
end

function MessageEvent:EventsToggle(aShow, aFrame)
	if string.lower(aFrame) == "all" then
		for i = 1, 7 do
			local frame = _G["ChatFrame"..i]
			if frame and not aShow and self:IsHooked(frame, "AddMessage") then
				self:Unhook(frame, "AddMessage")
			elseif frame and not self:IsHooked(frame, "AddMessage") then
				self:Hook(frame, "AddMessage", true)
			end
		end
	else
		local num = tonumber(aFrame)
		if not num then return end
		local frame = _G["ChatFrame"..num]
		if frame and not aShow and self:IsHooked(frame, "AddMessage") then
			self:Unhook(frame, "AddMessage")
		elseif frame and not self:IsHooked(frame, "AddMessage") then
			self:Hook(frame, "AddMessage")
		end
	end
end

function MessageEvent:AddMessage(frame, text, r, g, b, id)
	self.hooks[frame]["AddMessage"](
		frame,
		text.." ["..tostring(event).."]",
		r, g, b, id
	)
end

